@@ -1,6 +1,7 @@
 // cmdline_clean.cc
 //
 // Copyright (C) 2004, 2010 Daniel Burrows
+// Copyright (C) 2015-2016 Manuel A. Fernandez Montecelo
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -14,13 +15,15 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+// Boston, MA 02110-1301, USA.
 
 
 // Local includes:
 #include "cmdline_clean.h"
 
+#include "cmdline_util.h"
+
 #include "text_progress.h"
 #include "terminal.h"
 
@@ -55,7 +58,7 @@ int cmdline_clean(int argc, char *argv[], bool simulate)
 {
   const string archivedir = aptcfg->FindDir("Dir::Cache::archives");
 
-  _error->DumpErrors();
+  aptitude::cmdline::on_apt_errors_print_and_die();
 
   if(argc != 1)
     {
@@ -69,30 +72,19 @@ int cmdline_clean(int argc, char *argv[], bool simulate)
       return 0;
     }
 
-  // Lock the archive directory
-  FileFd lock;
-  if (_config->FindB("Debug::NoLocking",false) == false)
-    {
-      lock.Fd(GetLock(archivedir + "lock"));
-      if (_error->PendingError() == true)
-        _error->Error(_("Unable to lock the download directory"));
-    }
+  // do clean
+  bool result_ok = aptitude::apt::clean_cache_dir();
 
-  if(_error->PendingError())
+  // results
+  if (result_ok)
+    {
+      return 0;
+    }
+  else
     {
       _error->DumpErrors();
       return -1;
     }
-
-  pkgAcquire fetcher;
-  fetcher.Clean(archivedir);
-  fetcher.Clean(archivedir+"partial/");
-
-  int rval=_error->PendingError() ? -1 : 0;
-
-  _error->DumpErrors();
-
-  return rval;
 }
 
 // Shamelessly stolen from apt-get:
@@ -130,7 +122,7 @@ int cmdline_autoclean(int argc, char *argv[], bool simulate)
   const string archivedir = aptcfg->FindDir("Dir::Cache::archives");
   const std::shared_ptr<terminal_io> term = create_terminal();
 
-  _error->DumpErrors();
+  aptitude::cmdline::on_apt_errors_print_and_die();
 
   if(argc != 1)
     {
@@ -144,23 +136,20 @@ int cmdline_autoclean(int argc, char *argv[], bool simulate)
       _config->FindB("Debug::NoLocking",false) == false)
     {
       lock.Fd(GetLock(archivedir + "lock"));
-      if (_error->PendingError() == true)
+      if (_error->PendingError())
         {
           _error->Error(_("Unable to lock the download directory"));
-          _error->DumpErrors();
-          return -1;
+
+	  aptitude::cmdline::on_apt_errors_print_and_die();
         }
     }
 
   std::shared_ptr<OpProgress> progress = make_text_progress(false, term, term, term);
 
-  apt_init(progress.get(), false);
+  bool operation_needs_lock = true;
+  apt_init(progress.get(), false, operation_needs_lock, nullptr);
 
-  if(_error->PendingError())
-    {
-      _error->DumpErrors();
-      return -1;
-    }
+  aptitude::cmdline::on_apt_errors_print_and_die();
 
   LogCleaner cleaner(simulate);
   int rval=0;
@@ -169,7 +158,7 @@ int cmdline_autoclean(int argc, char *argv[], bool simulate)
      _error->PendingError())
     rval=-1;
 
-  _error->DumpErrors();
+  aptitude::cmdline::on_apt_errors_print_and_die();
 
   if(simulate)
     printf(_("Would free %sB of disk space\n"),
