@@ -1,6 +1,7 @@
 // cmdline_forget_new.cc
 //
 // Copyright (C) 2004, 2010 Daniel Burrows
+// Copyright (C) 2015-2016 Manuel A. Fernandez Montecelo
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -14,8 +15,8 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+// Boston, MA 02110-1301, USA.
 
 
 // Local includes:
@@ -26,8 +27,11 @@
 
 #include <aptitude.h>
 
+#include "cmdline_util.h"
 #include <generic/apt/apt.h>
 #include <generic/apt/config_signal.h>
+#include <generic/apt/matching/pattern.h>
+#include <generic/util/util.h>
 
 
 // System includes:
@@ -49,25 +53,13 @@ int cmdline_forget_new(int argc, char *argv[],
 {
   const std::shared_ptr<terminal_io> term = create_terminal();
 
-  _error->DumpErrors();
-
-  // NB: perhaps we should allow forgetting the new state of just
-  // a few packages?
-  if(argc != 1)
-    {
-      fprintf(stderr, _("E: The forget-new command takes no arguments\n"));
-      return -1;
-    }  
+  aptitude::cmdline::on_apt_errors_print_and_die();
 
   std::shared_ptr<OpProgress> progress = make_text_progress(false, term, term, term);
+  bool operation_needs_lock = true;
+  apt_init(progress.get(), false, operation_needs_lock, status_fname);
 
-  apt_init(progress.get(), false, status_fname);
-
-  if(_error->PendingError())
-    {
-      _error->DumpErrors();
-      return -1;
-    }
+  aptitude::cmdline::on_apt_errors_print_and_die();
 
   // In case we aren't root.
   if(!simulate)
@@ -75,27 +67,37 @@ int cmdline_forget_new(int argc, char *argv[],
   else
     apt_cache_file->ReleaseLock();
 
-  if(_error->PendingError())
-    {
-      _error->DumpErrors();
-      return -1;
-    }
+  aptitude::cmdline::on_apt_errors_print_and_die();
+
+  bool all_ok = true;
 
   if(simulate)
     printf(_("Would forget what packages are new\n"));
   else
     {
-      (*apt_cache_file)->forget_new(NULL);
-
-      (*apt_cache_file)->save_selection_list(*progress);
-
-      if(_error->PendingError())
+      std::vector<std::string> args;
+      int argc_start = 1;
+      for (int i = argc_start; i < argc; ++i)
 	{
-	  _error->DumpErrors();
-
-	  return -1;
+	  args.push_back(argv[i]);
 	}
+
+      std::vector<pkgCache::PkgIterator> pkg_its = aptitude::cmdline::get_packages_from_set_of_strings(args, all_ok);
+
+      // if the was some problem, stop here
+      if (!all_ok)
+	return -1;
+
+      // whether packages/patterns were given, or if it should forget all new
+      if (argc == 1)
+	(*apt_cache_file)->forget_new(NULL);
+      else
+	(*apt_cache_file)->forget_new(NULL, pkg_its);
+
+      (*apt_cache_file)->save_selection_list(progress.get());
+
+      aptitude::cmdline::on_apt_errors_print_and_die();
     }
 
-  return 0;
+  return (all_ok ? 0 : -1);
 }
