@@ -1,6 +1,7 @@
 // cmdline_util.cc
 //
 // Copyright (C) 2004, 2010 Daniel Burrows
+// Copyright (C) 2015-2016 Manuel A. Fernandez Montecelo
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -14,8 +15,8 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+// Boston, MA 02110-1301, USA.
 
 
 // Local includes:
@@ -95,6 +96,11 @@ void ui_preview()
 				      &result));
   do_package_run_or_show_preview();
   ui_main();
+
+  // copied from "struct close_cache_on_exit" in main.cc, to fix #817776 and
+  // #816322 before that in ui_solution_screen()
+  apt_shutdown();
+
   exit(result);
 }
 
@@ -108,6 +114,11 @@ void ui_solution_screen()
 
   do_examine_solution();
   ui_main();
+
+  // copied from "struct close_cache_on_exit" in main.cc, to fix #816322 and
+  // #817776 after that in ui_preview()
+  apt_shutdown();
+
   exit(0);
 }
 
@@ -464,7 +475,9 @@ download_manager::result cmdline_do_download(download_manager *m,
       OpProgress tmpProgress;
       // Dump errors so we don't spuriously think we failed.
       _error->DumpErrors();
-      apt_load_cache(&tmpProgress, false, NULL);
+      bool operation_needs_lock = true;
+      bool reset_reinstall = false;
+      apt_load_cache(&tmpProgress, false, operation_needs_lock, nullptr, reset_reinstall);
       initial_stats = compute_apt_stats();
     }
 
@@ -497,8 +510,7 @@ download_manager::result cmdline_do_download(download_manager *m,
   stats final_stats(0, 0, 0, std::set<std::string>());
   if(aptcfg->FindI("Quiet", 0) == 0)
     {
-      OpProgress tmpProgress;
-      apt_load_cache(&tmpProgress, false, NULL);
+      // cache just (re)loaded in last steps of download_manager
       final_stats = compute_apt_stats();
       show_stats_change(initial_stats, final_stats,
 			verbose >= 1, verbose >= 2,
@@ -680,7 +692,7 @@ namespace aptitude
 	      //
 	      // - defaults to package name, no need to check if it's empty
 	      source_package_name = ver.SourcePkgName();
-	      source_version = ver.SourceVerStr();
+	      std::string source_version = ver.SourceVerStr();
 
 	      rval = find_source_package(source_package_name, source_version);
 #else
@@ -818,5 +830,82 @@ namespace aptitude
 
       return output;
     }
+
+
+    std::vector<pkgCache::PkgIterator> get_packages_from_string(const std::string& str)
+    {
+      std::vector<pkgCache::PkgIterator> pkgs;
+
+      if (!aptitude::matching::is_pattern(str))
+	{
+	  pkgCache::PkgIterator pkg = (*apt_cache_file)->FindPkg(str);
+	  if (!pkg.end())
+	    {
+	      pkgs.push_back(pkg);
+	    }
+	}
+      else
+	{
+	  using namespace aptitude::matching;
+	  using cwidget::util::ref_ptr;
+
+	  ref_ptr<pattern> p(parse(str));
+
+	  if (p.valid())
+	    {
+	      std::vector<std::pair<pkgCache::PkgIterator, ref_ptr<structural_match> > > matches;
+	      ref_ptr<search_cache> search_info(search_cache::create());
+	      search(p, search_info,
+		     matches,
+		     *apt_cache_file,
+		     *apt_package_records);
+
+	      for (std::vector<std::pair<pkgCache::PkgIterator, ref_ptr<structural_match>>>::const_iterator
+		     it = matches.begin(); it != matches.end(); ++it)
+		{
+		  const pkgCache::PkgIterator& pkg = it->first;
+		  pkgs.push_back(pkg);
+		}
+	    }
+	}
+
+      return pkgs;
+    }
+
+    std::vector<pkgCache::PkgIterator> get_packages_from_set_of_strings(const std::vector<std::string>& args, bool& all_ok)
+    {
+      std::vector<pkgCache::PkgIterator> pkgs;
+
+      for (const std::string& arg : args)
+	{
+	  std::vector<pkgCache::PkgIterator> pkgs_from_args = aptitude::cmdline::get_packages_from_string(arg);
+
+	  if (pkgs_from_args.empty())
+	    {
+	      // problem parsing command line or finding packages
+
+	      all_ok = false;
+
+	      int quiet = aptcfg->FindI("quiet", 0);
+	      if (quiet == 0)
+		{
+		  if (aptitude::matching::is_pattern(arg.c_str()))
+		    std::cerr << cw::util::ssprintf(_("No packages match pattern \"%s\""), arg.c_str()) << std::endl;
+		  else
+		    std::cerr << cw::util::ssprintf(_("No such package \"%s\""), arg.c_str()) << std::endl;
+		}
+	    }
+	  else
+	    {
+	      // append packages
+	      for (const auto& it : pkgs_from_args)
+		{
+		  pkgs.push_back(it);
+		}
+	    }
+	}
+
+      return pkgs;
+    }
   }
 }
