@@ -14,8 +14,8 @@
 //
 //   You should have received a copy of the GNU General Public License
 //   along with this program; see the file COPYING.  If not, write to
-//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-//   Boston, MA 02111-1307, USA.
+//   the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+//   Boston, MA 02110-1301, USA.
 
 #ifndef HISTORY_ENTRY_H
 #define HISTORY_ENTRY_H
@@ -25,6 +25,7 @@
 #include <apt-pkg/pkgcache.h> // For enumerations.
 
 #include <generic/apt/aptcache.h>
+#include <generic/apt/usertags.h>
 
 #include <cwidget/generic/util/exception.h>
 #include <cwidget/generic/util/ref_ptr.h>
@@ -638,7 +639,7 @@ namespace aptitude
 
       std::vector<resolver_action> resolver_actions;
 
-      std::set<aptitudeDepCache::user_tag> old_user_tags, new_user_tags;
+      std::set<user_tag> old_user_tags, new_user_tags;
 
       std::vector<cwidget::util::ref_ptr<entry> > sub_entries;
 
