@@ -1,6 +1,7 @@
 // pkg_acqfile.h                  -*-c++-*-
 //
 //   Copyright (C) 2002, 2005 Daniel Burrows
+//   Copyright 2015 Manuel A. Fernandez Montecelo
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -14,8 +15,12 @@
 //
 //   You should have received a copy of the GNU General Public License
 //   along with this program; see the file COPYING.  If not, write to
-//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-//   Boston, MA 02111-1307, USA.
+//   the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+//   Boston, MA 02110-1301, USA.
+
+
+#ifndef PKG_ACQFILE_H
+#define PKG_ACQFILE_H
 
 #include <apt-pkg/acquire-item.h>
 
@@ -28,3 +33,5 @@
 bool get_archive(pkgAcquire *Owner, pkgSourceList *Sources,
 		 pkgRecords *Recs, pkgCache::VerIterator const &Version,
 		 std::string directory, std::string &StoreFilename);
+
+#endif
