@@ -1,7 +1,7 @@
 // temp.h                                 -*-c++-*-
 //
 //   Copyright (C) 2005, 2007-2010 Daniel Burrows
-//   Copyright (C) 2015 Manuel A. Fernandez Montecelo
+//   Copyright (C) 2015-2016 Manuel A. Fernandez Montecelo
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -15,8 +15,8 @@
 //
 //   You should have received a copy of the GNU General Public License
 //   along with this program; see the file COPYING.  If not, write to
-//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-//   Boston, MA 02111-1307, USA.
+//   the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+//   Boston, MA 02110-1301, USA.
 //
 
 #ifndef TEMP_H
@@ -270,10 +270,14 @@ namespace temp
 
     bool valid() const;
     std::string get_name() const;
+    std::string get_base_dir() const;
   };
 
   class name::impl
   {
+    /** The parent/base dir of this temporary object. */
+    std::string parentdir;
+
     /** The name of this temporary object. */
     std::string filename;
 
@@ -301,6 +305,9 @@ namespace temp
       return filename;
     }
 
+    /** \return the temporary's base dir. */
+    std::string get_base_dir() const;
+
     /** Increment the reference count of this impl. */
     void incref()
     {
@@ -371,6 +378,11 @@ namespace temp
   {
     return real_name->get_name();
   }
+
+  inline std::string name::get_base_dir() const
+  {
+    return real_name->get_base_dir();
+  }
 };
 
 #endif // TEMP_H
