@@ -1,32 +1,33 @@
 // load_grouppolicy.cc
 //
 //  Copyright 2001 Daniel Burrows
-//
-//  Routines to parse grouping policies
-//
-// This uses a bunch of little functions to parse stuff.  Sue me.
-//
-// The parsers take an arglist and a chain argument, and return either a
-// new pkg_grouppolicy_factory, or (if an error was encountered) NULL.
-// (note: the chain argument will be NULL if the policy is the tail of a list)
-//
-// The astute observer will wonder why I don't just use bison or
-// something similar.  The main reason is also the main reason I can't
-// just do a straightforward parse: the matcher grammar.
-// Unfortunately, it's hard to assemble grammars modularly in yacc,
-// and I don't want to duplicate the matchers' grammar in several
-// places; there's also the "minor" problem that the lexical analysis
-// of matchers is radically different from the rest of the constructs.
-//
-// TODO: a lot of classes just exist to complain about having
-// arguments and then creating a new instance of a policy0;
-// commonalize this.
+//  Copyright 2015 Manuel A. Fernandez Montecelo
+
+
+///  Routines to parse grouping policies
+///
+/// This uses a bunch of little functions to parse stuff.  Sue me.
+///
+/// The parsers take an arglist and a chain argument, and return either a
+/// new pkg_grouppolicy_factory, or (if an error was encountered) NULL.
+/// (note: the chain argument will be NULL if the policy is the tail of a list)
+///
+/// The astute observer will wonder why I don't just use bison or
+/// something similar.  The main reason is also the main reason I can't
+/// just do a straightforward parse: the matcher grammar.
+/// Unfortunately, it's hard to assemble grammars modularly in yacc,
+/// and I don't want to duplicate the matchers' grammar in several
+/// places; there's also the "minor" problem that the lexical analysis
+/// of matchers is radically different from the rest of the constructs.
+///
+/// TODO: a lot of classes just exist to complain about having
+/// arguments and then creating a new instance of a policy0;
+/// commonalize this.
 
 #include "load_grouppolicy.h"
 
 #include <generic/apt/matching/parse.h>
 #include <generic/apt/matching/pattern.h>
-#include <generic/apt/pkg_hier.h>
 
 #include <generic/util/util.h>
 
@@ -265,7 +266,7 @@ public:
 				 const string::const_iterator &end)
   {
     string name;
-    auto_ptr<group_policy_parse_node> rval(NULL);
+    unique_ptr<group_policy_parse_node> rval(nullptr);
 
     while(begin != end)
       {
@@ -318,24 +319,24 @@ public:
 					name.c_str());
 	    else
 	      {
-		auto_ptr<group_policy_parse_node> curr(found->second->parse(begin, end));
+		unique_ptr<group_policy_parse_node> curr(found->second->parse(begin, end));
 
-		if(rval.get() != NULL && rval.get()->terminal())
+		if(rval.get() != nullptr && rval.get()->terminal())
 		  throw GroupParseException(_("Terminal policy '%s' should be the last policy in the list"), last.c_str());
 
 
-		if(rval.get() == NULL)
-		  rval = curr;
+		if(rval.get() == nullptr)
+		  rval.swap(curr);
 		else
-		  rval = auto_ptr<group_policy_parse_node>(new group_policy_pair_node(rval.release(), curr.release()));
+		  rval = unique_ptr<group_policy_parse_node>(new group_policy_pair_node(rval.release(), curr.release()));
 	      }
 	  }
       }
 
-    if(rval.get() == NULL)
-      rval = auto_ptr<group_policy_parse_node>(new group_policy_end_node);
+    if(rval.get() == nullptr)
+      rval = unique_ptr<group_policy_parse_node>(new group_policy_end_node);
     else if(!rval->terminal())
-      rval = auto_ptr<group_policy_parse_node>(new group_policy_pair_node(rval.release(), new group_policy_end_node));
+      rval = unique_ptr<group_policy_parse_node>(new group_policy_pair_node(rval.release(), new group_policy_end_node));
 
     eassert(rval->terminal());
     return rval.release();
@@ -528,10 +529,35 @@ class firstchar_policy_parser : public string_policy_parser
 {
   group_policy_parse_node *create_node(const vector<string> &args)
   {
-    if(args.size()!=0)
-      throw GroupParseException(_("Grouping policy '%s' takes no arguments"), "firstchar");
+    typedef pkg_grouppolicy_firstchar_factory::package_name_mode_type package_name_mode_type;
 
-    return new policy_node0<pkg_grouppolicy_firstchar_factory>;
+    package_name_mode_type package_name_mode = package_name_mode_type::binary;
+
+    if (args.size() == 0)
+      {
+	// OK, default to ::binary
+      }
+    else if (args.size() == 1)
+      {
+	if (args[0] == "binary")
+	  {
+	    // OK, default to ::binary
+	  }
+	else if (args[0] == "source")
+	  {
+	    package_name_mode = package_name_mode_type::source;
+	  }
+	else
+	  {
+	    throw GroupParseException(_("Bad mode name '%s' (use 'binary' or 'source')"), args[0].c_str());
+	  }
+      }
+    else
+      {
+	throw GroupParseException(_("Too many arguments to '%s' grouping policy"), "firstchar");
+      }
+
+    return new policy_node1<pkg_grouppolicy_firstchar_factory, package_name_mode_type>(package_name_mode);
   }
 };
 
@@ -541,7 +567,7 @@ public:
   group_policy_parse_node *create_node(const vector<string> &args)
   {
     if(args.size()!=0)
-      throw GroupParseException(_("Too many arguments to '%s' grouping policy"), "source");
+      throw GroupParseException(_("Grouping policy '%s' takes no arguments"), "source");
 
     return new policy_node0<pkg_grouppolicy_source_factory>;
   }
@@ -580,26 +606,6 @@ class priority_policy_parser : public string_policy_parser
   }
 };
 
-class hier_policy_parser : public string_policy_parser
-{
-  group_policy_parse_node *create_node(const vector<string> &args)
-  {
-    if(!args.empty())
-      {
-	// FIXME: who deletes this??
-	pkg_hier *hier=new pkg_hier;
-
-	for(vector<string>::const_iterator i = args.begin();
-	    i != args.end(); ++i)
-	  hier->input_file(*i);
-
-	return new policy_node2<pkg_grouppolicy_hier_factory, pkg_hier *, bool>(hier, true);
-      }
-    else
-      return new policy_node2<pkg_grouppolicy_hier_factory, pkg_hier *, bool>(get_user_pkg_hier(), false);
-  }
-};
-
 class task_policy_parser : public string_policy_parser
 {
   group_policy_parse_node *create_node(const vector<string> &args)
@@ -661,7 +667,7 @@ class pattern_policy_parser : public group_policy_parser
 								     true, false, false));
 
 	bool passthrough = false;
-	std::auto_ptr<pkg_grouppolicy_factory> chain;
+	std::unique_ptr<pkg_grouppolicy_factory> chain;
 
 	if(!pattern.valid())
 	  throw GroupParseException(_("Unable to parse pattern after \"%s\""),
@@ -702,7 +708,7 @@ class pattern_policy_parser : public group_policy_parser
 	    stripws(format);
 
 	    if(format.empty())
-	      throw GroupParseException(_("Unexpectedly empty tree title after \"%s\""),
+	      throw GroupParseException(_("Unexpected empty tree title after \"%s\""),
 					string(begin0, end).c_str());
 	  }
 	else if(begin != end && *begin == '|')
@@ -734,7 +740,7 @@ class pattern_policy_parser : public group_policy_parser
 
 	    list_policy_parser subpolicy_tail_parser(parse_types, '}');
 
-	    std::auto_ptr<group_policy_parse_node>
+	    std::unique_ptr<group_policy_parse_node>
 	      sub_parse_node(subpolicy_tail_parser.parse(begin, end));
 
 	    chain.reset(sub_parse_node->instantiate(NULL));
@@ -786,7 +792,6 @@ static void init_parse_types()
       parse_types["versions"]=new ver_policy_parser;
       parse_types["deps"]=new dep_policy_parser;
       parse_types["priority"]=new priority_policy_parser;
-      parse_types["hier"]=new hier_policy_parser;
       parse_types["task"]=new task_policy_parser;
       parse_types["tag"]=new tag_policy_parser;
 
@@ -804,7 +809,7 @@ pkg_grouppolicy_factory *parse_grouppolicy(const string &s)
 
   try
     {
-      auto_ptr<group_policy_parse_node> node(list_policy_parser(parse_types).parse(begin, s.end()));
+      unique_ptr<group_policy_parse_node> node(list_policy_parser(parse_types).parse(begin, s.end()));
 
       eassert(begin == s.end());
 
