@@ -3,7 +3,7 @@
 //  The pkg_columnizer class.
 //
 //  Copyright 1999-2005, 2007-2008, 2010 Daniel Burrows
-//  Copyright 2012-2015 Manuel A. Fernandez Montecelo
+//  Copyright 2012-2016 Manuel A. Fernandez Montecelo
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; see the file COPYING.  If not, write to
-//  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-//  Boston, MA 02111-1307, USA.
+//  the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+//  Boston, MA 02110-1301, USA.
 
 #include "aptitude.h"
 
@@ -64,27 +64,29 @@ cw::config::column_type_defaults pkg_item::pkg_columnizer::defaults[pkg_columniz
   {10, false, true},    // section
   {2, false, false},    // revdepcount
   {1, false, false},    // autoset
-  {1, false, false},    // tagged
+  {30, false, false},   // tagged (also user-tags)
+  {30, false, false},   // source
+  {10, false, false},   // architecture
+  {30, false, false},   // origin
   {10, true, true},     // archive
   {9, false, false},    // sizechange
   {strlen(PACKAGE), false, false},  // progname
   {strlen(VERSION), false, false},  // progver
-  {12, false, true},    // brokencount
-  {30, false, true},    // diskusage
-  {17, false, true},    // downloadsize
+  {12, false, false},   // brokencount
+  {16, false, false},   // diskusage, example (max): "Disk: -2,587 kB "
+  {22, false, false},   // downloadsize, example (max): " DL: 2,586 kB/2,587 kB"
   {4, false, false},    // pin_priority
-  {15, false, true},    // hostname
+  {8, true, false},     // hostname
   {1, false, false}     // trust_state
 };
 
-// Default widths for:
-// name, installed_size, debsize, stateflag, actionflag, description, currver,
-// candver, longstate, longaction, maintainer, priority, section, revdepcount,
-// brokencount, diskusage, downloadsize.
+// Default widths for columns: name, installed_size, debsize... etc.
 //
-// You can't set default widths for the program name and version here (those
-// strings aren't affected by translation, for one thing)
-const char *default_widths = N_("30 8 8 1 1 40 14 14 11 10 35 9 10 2 1 1 10 9 12 30 17 15");
+// Complete list and order in "pkg_item::pkg_columnizer::defaults", except that
+// "progname" and "progver" are fixed (not affected by translation), and that
+// shortpriority, pin_priority and trust_state have also fixed sizes due to
+// being of numerical nature or size requirements that translators sould respect
+const char *default_widths = N_("30 8 8 1 1 40 14 14 11 10 35 9 10 2 1 30 30 10 30 10 9 12 16 22 8");
 
 const char *pkg_item::pkg_columnizer::column_names[pkg_columnizer::numtypes]=
   {N_("Package"),
@@ -102,7 +104,10 @@ const char *pkg_item::pkg_columnizer::column_names[pkg_columnizer::numtypes]=
    N_("Section"),
    N_("RC"),
    N_("Auto"),
-   N_("Tag"),
+   N_("Tag/user-tags"),
+   N_("Source"),
+   N_("Architecture"),
+   N_("Origin"),
 
    // These don't make sense with headers, but whatever:
    N_("ProgName"),
@@ -345,34 +350,22 @@ cw::column_disposition pkg_item::pkg_columnizer::setup_column(const pkgCache::Pk
 
       break;
     case priority:
-      if(!visible_ver.end() &&
-	 const_cast<pkgCache::VerIterator &>(visible_ver).PriorityType() &&
-	 const_cast<pkgCache::VerIterator &>(visible_ver).PriorityType()[0])
-	return cw::column_disposition(const_cast<pkgCache::VerIterator &>(visible_ver).PriorityType(), 0);
-      else
-	return cw::column_disposition(_("Unknown"), 0);
-
-      break;
     case shortpriority:
-      if(!visible_ver.end())
-	switch(visible_ver->Priority)
-	  {
-	  case pkgCache::State::Important:
-	    return cw::column_disposition(_("Imp"), 0);
-	  case pkgCache::State::Required:
-	    return cw::column_disposition(_("Req"), 0);
-	  case pkgCache::State::Standard:
-	    return cw::column_disposition(_("Std"), 0);
-	  case pkgCache::State::Optional:
-	    return cw::column_disposition(_("Opt"), 0);
-	  case pkgCache::State::Extra:
-	    return cw::column_disposition(_("Xtr"), 0);
-	  default:
-	    return cw::column_disposition(_("ERR"), 0);
-	  }
-      else
-	return cw::column_disposition(_("Unknown"), 0);
+      {
+	bool short_form = (type == shortpriority) ? true : false;
+	std::string priority_str;
 
+	if (!visible_ver.end())
+	  {
+	    priority_str = aptitude::apt::priority_to_string(static_cast<pkgCache::State::VerPriority>(visible_ver->Priority), short_form);
+	  }
+	else
+	  {
+	    priority_str = _("Unknown");
+	  }
+
+	return cw::column_disposition(priority_str, 0);
+      }
       break;
     case section:
       if(!visible_ver.end() && visible_ver.Section())
@@ -383,8 +376,10 @@ cw::column_disposition pkg_item::pkg_columnizer::setup_column(const pkgCache::Pk
       break;
     case progname:
       return cw::column_disposition(PACKAGE, 0);
+      break;
     case progver:
       return cw::column_disposition(VERSION, 0);
+      break;
     case brokencount:
       if(apt_cache_file && (*apt_cache_file)->BrokenCount()>0)
 	{
@@ -400,24 +395,14 @@ cw::column_disposition pkg_item::pkg_columnizer::setup_column(const pkgCache::Pk
       break;
     case diskusage:
       {
-	char buf[256];
-	if(apt_cache_file && (*apt_cache_file)->UsrSize()>0)
+	size_t bufsize = 256;
+	char buf[bufsize] = "";
+	if (apt_cache_file && ((*apt_cache_file)->UsrSize() != 0))
 	  {
-	    // Be translator-friendly -- breaking messages up like that is not
-	    // so good..
-	    snprintf(buf, 256, _("Will use %sB of disk space"), SizeToStr((*apt_cache_file)->UsrSize()).c_str());
-	    return cw::column_disposition(buf, 0);
+	    char sign = ((*apt_cache_file)->UsrSize() > 0) ? '+' : '-';
+	    snprintf(buf, bufsize, _("Disk: %c%sB"), sign, SizeToStr((*apt_cache_file)->UsrSize()).c_str());
 	  }
-	else if(apt_cache_file &&
-		(*apt_cache_file)->UsrSize()<0)
-	  {
-	    // Be translator-friendly -- breaking messages up like that is not
-	    // so good..
-	    snprintf(buf, 256, _("Will free %sB of disk space"), SizeToStr(-(*apt_cache_file)->UsrSize()).c_str());
-	    return cw::column_disposition(buf, 0);
-	  }
-	else
-	  return cw::column_disposition("", 0);
+	return cw::column_disposition(buf, 0);
       }
 
       break;
@@ -425,10 +410,28 @@ cw::column_disposition pkg_item::pkg_columnizer::setup_column(const pkgCache::Pk
       {
 	if(apt_cache_file && (*apt_cache_file)->DebSize()!=0)
 	  {
+	    auto f = aptitude::apt::get_pkgAcquire_fetch_info();
 	    char buf[256];
-	    snprintf(buf, 256,
-		     _("DL Size: %sB"), SizeToStr((*apt_cache_file)->DebSize()).c_str());
-	    return cw::column_disposition(buf, 0);
+	    if (f && (f->FetchNeeded != f->TotalNeeded))
+	      {
+		snprintf(buf, 256, _("DL: %sB/%sB"),
+			 SizeToStr(f->FetchNeeded).c_str(),
+			 SizeToStr(f->TotalNeeded).c_str());
+	      }
+	    else
+	      {
+		snprintf(buf, 256, _("DL: %sB"),
+			 SizeToStr((*apt_cache_file)->DebSize()).c_str());
+	      }
+
+	    // align to the right
+	    string s(buf);
+	    size_t total_size = defaults[downloadsize].width;
+	    int fill_size = (total_size - s.size());
+	    if (fill_size > 0)
+	      s.insert(0, fill_size, ' ');
+
+	    return cw::column_disposition(s.c_str(), 0);
 	  }
 	else
 	  return cw::column_disposition("", 0);
@@ -437,28 +440,27 @@ cw::column_disposition pkg_item::pkg_columnizer::setup_column(const pkgCache::Pk
       break;
     case pin_priority:
       {
-	if(apt_cache_file && !pkg.end())
+	// sibling implementation of the one in pkg_ver_item.cc
+
+	// empty by default
+	string pin_priority_str;
+
+	pkgPolicy* policy = dynamic_cast<pkgPolicy *>(&(*apt_cache_file)->GetPolicy());
+
+	if (apt_cache_file && policy && !pkg.end())
 	  {
-	    char buf[256];
+	    signed short priority = policy->GetPriority(pkg);
 
-	    pkgPolicy *policy=dynamic_cast<pkgPolicy *>(&(*apt_cache_file)->GetPolicy());
-
-	    if(!policy)
-	      return cw::column_disposition("", 0);
-
-	    // Not quite sure what this indicates
-	    signed short priority=policy->GetPriority(pkg);
-
-	    if(priority==0)
-	      return cw::column_disposition("", 0);
-
-	    snprintf(buf, 256, "%d", priority);
-
-	    return cw::column_disposition(buf, 0);
+	    if (priority != 0)
+	      {
+		pin_priority_str = std::to_string(priority);
+	      }
 	  }
-	else
-	  return cw::column_disposition("", 0);
+
+	// return whatever was gathered
+	return cw::column_disposition(pin_priority_str, 0);
       }
+      break;
     case autoset:
       // Display the "auto" flag UNLESS the package has been removed
       // or purged already and is not presently being installed.
@@ -472,22 +474,87 @@ cw::column_disposition pkg_item::pkg_columnizer::setup_column(const pkgCache::Pk
 
       break;
     case tagged:
-      if(!pkg.end() && (*apt_cache_file)->get_ext_state(pkg).tagged)
-	return cw::column_disposition("*", 0);
-      else
-	return cw::column_disposition("", 0);
+      {
+	string tagged_str = "";
 
+	// old "tagged" (actually unused)
+	if (!pkg.end() && (*apt_cache_file)->get_ext_state(pkg).tagged)
+	  tagged_str = "*";
+
+	// reuse field also for user-tags (some parts of #498442 and #665824)
+	for (const string& s : (*apt_cache_file)->get_user_tags(pkg))
+	  {
+	    if (tagged_str == "*")
+	      tagged_str += " ";
+	    else if (!tagged_str.empty())
+	      tagged_str += ", ";
+
+	    tagged_str += s;
+	  }
+
+	return cw::column_disposition(tagged_str, 0);
+      }
+      break;
+
+    case source:
+      {
+	std::string source_package_str;
+#if APT_PKG_MAJOR >= 5
+	// with apt-1.1:
+	//
+	// - SourcePkg (and Version) are in the binary cache and available via
+	//   the VerIterator; much faster than parsing the pkgRecord
+	//
+	// - defaults to package name, no need to check if it's empty
+	if ( !visible_ver.end() )
+	  {
+	    source_package_str = visible_ver.SourcePkgName();
+	  }
+#else
+	pkgRecords::Parser &rec=apt_package_records->Lookup(visible_ver.FileList());
+	source_package_str = rec.SourcePkg().empty() ? pkg.Name() : rec.SourcePkg();
+#endif
+
+	return cw::column_disposition(source_package_str, 0);
+      }
+      break;
+
+    case architecture:
+      {
+	std::string architecture_str;
+	if ( !visible_ver.end() )
+	  {
+	    architecture_str = visible_ver.Arch();
+	  }
+
+	return cw::column_disposition(architecture_str, 0);
+      }
+      break;
+
+    case origin:
+      {
+	std::string origin_str = get_origin(visible_ver, apt_package_records);
+	return cw::column_disposition(origin_str, 0);
+      }
       break;
 
     case archive:
       if(!visible_ver.end())
 	{
-	  string buf = archives_text(visible_ver, true, ",");
-	  return cw::column_disposition(buf,0);
+	  bool ignore_local = true;
+	  string archives = archives_text(visible_ver, ignore_local, ",");
+	  // see #349413 -- say 'now' for obsolete/local installed packages only
+	  if (archives.empty())
+	    {
+	      ignore_local = false;
+	      archives = archives_text(visible_ver, ignore_local, ",");
+	    }
+	  return cw::column_disposition(archives, 0);
 	}
       else
 	return cw::column_disposition("", 0);
 
+      break;
     case hostname:
       {
 	char buf[256];
@@ -506,13 +573,13 @@ cw::column_disposition pkg_item::pkg_columnizer::setup_column(const pkgCache::Pk
 	int count=0;
 	char buf[100];
 
-        if(pkg.end())
+        if (pkg.end())
           return cw::column_disposition("", 0);
 
-	if(!visible_ver.end())
+	if (!visible_ver.end())
 	  {
-	    for(pkgCache::DepIterator D=pkg.RevDependsList(); !D.end(); D++)
-	      if(D.IsCritical() &&
+	    for (pkgCache::DepIterator D=pkg.RevDependsList(); !D.end(); ++D)
+	      if (D.IsCritical() &&
 		 !is_conflict(D->Type) &&
 		 D.ParentVer()==D.ParentPkg().CurrentVer() &&
 		 // That test is CORRECT; we want to see if the version
@@ -521,18 +588,23 @@ cw::column_disposition pkg_item::pkg_columnizer::setup_column(const pkgCache::Pk
 		 //  and couldn't remember what it did despite writing it
 		 //  5 minutes ago. Maybe I should have my head examined :) )
 		 _system->VS->CheckDep(visible_ver.VerStr(), D->CompareOp, D.TargetVer()))
-		count++;
+		{
+		  ++count;
+		}
+
+	    for (pkgCache::PrvIterator i=visible_ver.ProvidesList(); !i.end(); ++i)
+	      for (pkgCache::DepIterator D=i.ParentPkg().RevDependsList(); !D.end(); ++D)
+		{
+		  if (D.IsCritical() &&
+		     !is_conflict(D->Type) &&
+		     D.ParentVer()==D.ParentPkg().CurrentVer() &&
+		     _system->VS->CheckDep(i.ProvideVersion(), D->CompareOp, D.TargetVer()))
+		    {
+		      ++count;
+		    }
+		}
 	  }
 
-	for(pkgCache::PrvIterator i=pkg.ProvidesList(); !i.end(); i++)
-	  for(pkgCache::DepIterator D=i.ParentPkg().RevDependsList(); !D.end(); D++)
-	    {
-	      if(D.IsCritical() &&
-		 !is_conflict(D->Type) &&
-		 D.ParentVer()==D.ParentPkg().CurrentVer() &&
-		 _system->VS->CheckDep(i.ProvideVersion(), D->CompareOp, D.TargetVer()))
-		count++;
-	    }
 	snprintf(buf, 100, "%i", count);
 	return cw::column_disposition(buf, 0);
       }
@@ -546,6 +618,7 @@ cw::column_disposition pkg_item::pkg_columnizer::setup_column(const pkgCache::Pk
 	else
 	  return cw::column_disposition("U", 0);
       }
+      break;
     default:
       return cw::column_disposition(_("ERROR"), 0);
     }
@@ -594,6 +667,12 @@ int pkg_item::pkg_columnizer::parse_column_type(char id)
       return tagged;
     case 't': // like apt-get -t
       return archive;
+    case 'E':
+      return architecture;
+    case 'e':
+      return source;
+    case 'O':
+      return origin;
 
     case 'B':
       return brokencount;
@@ -631,7 +710,7 @@ public:
 void pkg_item::pkg_columnizer::init_formatting()
 {
   sscanf(_(default_widths),
-	 "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
+	 "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
 	 &defaults[name].width,
 	 &defaults[installed_size].width,
 	 &defaults[debsize].width,
@@ -648,6 +727,9 @@ void pkg_item::pkg_columnizer::init_formatting()
 	 &defaults[revdepcount].width,
 	 &defaults[autoset].width,
 	 &defaults[tagged].width,
+	 &defaults[source].width,
+	 &defaults[architecture].width,
+	 &defaults[origin].width,
 	 &defaults[archive].width,
 	 &defaults[sizechange].width,
 	 &defaults[brokencount].width,
